#undef OPENTRACKER_EXPORTS
#include <OpenTracker/OpenTracker.h>
#include <OpenTracker/input/SPLModules.h>
#include <OpenTracker/input/MyTutorialModule.h>
#include <iostream>
#include <fstream>
#include <exception>
#include <stdlib.h>
#include <string.h>

#include <OpenTracker/types/Image.h>

#include "pgmimage.h"
#ifdef _HAVE_SIMPLE_CANVAS
 #include "simplecanvas.h"
#endif //_HAVE_SIMPLE_CANVAS

#define MAX_NUM_IMAGE 20

using namespace std;
using namespace ot;

#define DEFAULT_LOOP_INTERVAL    10   //ms

int main(int argc, char **argv)
{
  char* xmlfile;
  long  interval;

  // set parameters based on arguments
  if (argc <= 1) {
    cerr << "Usage: " << argv[0] << " <OT XML> [<interval in ms>]"  << endl;
    exit(-1);
  }
  xmlfile = argv[1];
  if (argc > 2) {
    interval = atoi(argv[2]);
  } else {
    interval = DEFAULT_LOOP_INTERVAL;
  }
  cerr << "XML File     : " << xmlfile << endl;
  cerr << "Loop interval: " << interval << "ms" << endl;


  // context loop
  addSPLModules();

  Context context(1);
  context.parseConfiguration(xmlfile);
  context.start();

  MyTutorialModule* mtm;
  MyTutorialSink*   mts;

  mtm = (MyTutorialModule*) context.getModule("MyTutorialConfig");
  if (mtm == NULL) {
    cerr << "MyTutorialModule has not been loaded." << endl;
    exit(-1);
  }
  mts = mtm->getSink();
  if (mts == NULL) {
    cerr << "MyTutorialSink has not been loaded." << endl;
    exit(-1);
  }

  int stopflag = 0;
  int i = 0;

#ifdef _HAVE_SIMPLE_CANVAS
  SimpleCanvas sc(256, 256);
  sc.run();
#endif //_HAVE_SIMPLE_CANVAS

  double prev_timestamp = 0.0;
  while (stopflag == 0) {

    stopflag = context.loopOnce();

    if (mts) {
      Image img;
      int w, h;
      
      if (mts->getTimestamp() > prev_timestamp) {
        mts->getImage(img, w, h);
        //std::cerr << "size: " << w << ", " << h << std::endl;
        if (w > 0 && h > 0 && img.size() != 0) {
          
          PgmImage pi(w, h, (short*)img.image_ptr);
          
#ifdef _HAVE_SIMPLE_CANVAS        
          sc.load((unsigned char*)pi.getDataC());
          sc.update();
#else
          char filename[50];
          sprintf(filename, "out-%03d.pgm", i);
          std::ofstream fout(filename, std::ios::binary);
          fout << pi;
          fout.close();
#endif //_HAVE_SIMPLE_CANVAS
          i ++;
          i = i % MAX_NUM_IMAGE;
        }
      }
    }

    OSUtils::sleep(interval);
  }

#ifdef _HAVE_SIMPLE_CANVAS        
  sc.stop();
#endif
  context.close();

}
