#undef OPENTRACKER_EXPORTS
#include <OpenTracker/OpenTracker.h>
#include <OpenTracker/input/SPLModules.h>
#include <OpenTracker/input/MyTutorialModule.h>
#include <iostream>
#include <exception>
#include <stdlib.h>

using namespace std;
using namespace ot;


#define DEFAULT_LOOP_INTERVAL    100 //ms

int main(int argc, char **argv)
{
  char* xmlfile;
  long  interval;

  // set parameters based on arguments
  if (argc <= 1) {
    cerr << "Usage: " << argv[0] << " <OT XML> [<interval in ms>]"  << endl;
    exit(-1);
  }
  xmlfile = argv[1];
  if (argc > 2) {
    interval = atoi(argv[2]);
  } else {
    interval = DEFAULT_LOOP_INTERVAL;
  }
  cerr << "XML File : " << xmlfile << endl;
  cerr << "Loop interval: " << (interval) << "ms" << endl;


  // context loop

  addSPLModules();

  Context context(1);
  context.parseConfiguration(xmlfile);
  context.start();

  MyTutorialModule* mtm = (MyTutorialModule*) context.getModule("MyTutorialConfig");

  std::vector<float> pos(3,0.0);
  std::vector<float> quat(4,0.0);

  pos[0] = 0.0;
  pos[1] = 1.0;
  pos[2] = 2.0;

  quat[0] = 0.0;
  quat[1] = 1.0;
  quat[2] = 2.0;
  quat[3] = 3.0;

  int stopflag = 0;

  while (stopflag == 0) {
    pos[0] += 0.0;
    pos[1] += 1.0;
    pos[2] += 2.0;
    quat[0] += 0.0;
    quat[1] += 1.0;
    quat[2] += 2.0;
    quat[3] += 3.0;
    std::cout << "Sending position = ("
              << pos[0] << ", " << pos[1] << ", " << pos[2] << "),  ";
    std::cout << "orientation = ("
              << quat[0] << ", " << quat[1] << ", " << quat[2] << ","
              << quat[3] << ")"  << std::endl;
    if (mtm) {
      mtm->setTracker(pos, quat);
    }

    stopflag = context.loopOnce();
    OSUtils::sleep(interval);
  }

  context.close();

}
