/*============================================================*
 *                                                            *
 * Simple Image Canvas on GTK+                                *
 *                                                            *
 * Copyright (C) 2007, Junichi Tokuda.                        *
 *                                                            *
 *============================================================*
 * Description: Simple Image Display class		      *
 * Created on:  10 May, 2007				      *
 *============================================================*/

#include <gdk/gdk.h>
#include <glib.h>
#include <pthread.h>
#include "simplecanvas.h"


SimpleCanvas* SimpleCanvas::pointer;

SimpleCanvas::SimpleCanvas(int w, int h)
{

  pointer      = this;

  this->width  = w;
  this->height = h;

  drawbuf      = new guchar[width*height];

  g_thread_init(NULL);
  gdk_threads_init();
}


SimpleCanvas::~SimpleCanvas()
{
}

gint SimpleCanvas::onDraw(GtkWidget *widget,GdkEventExpose *event)
{
  if (pointer->drawbuf != NULL) {
    gdk_draw_gray_image(widget->window, widget->style->fg_gc[GTK_STATE_NORMAL],
			0, 0, pointer->width, pointer->height,
			GDK_RGB_DITHER_MAX, pointer->drawbuf, pointer->width);
  }
  return FALSE;
}


void SimpleCanvas::load(short* ptr)
{
  gdk_threads_enter();
  guchar *pend = drawbuf + width*height;
  guchar *p = drawbuf;

  while (p < pend) {
    *p = *ptr;
    p   ++;
    ptr ++;
  }
  gdk_threads_leave();
}


void SimpleCanvas::load(unsigned char* ptr)
{
  gdk_threads_enter();
  guchar *pend = drawbuf + width*height;
  guchar *p = drawbuf;

  while (p < pend) {
    *p = *ptr;
    p   ++;
    ptr ++;
  }
  gdk_threads_leave();
}


void SimpleCanvas::update()
{
  gdk_threads_enter();
  if (pointer->drawbuf != NULL) {
    gdk_draw_gray_image(GTK_WIDGET(drawing_area)->window, 
			pointer->drawing_area->style->fg_gc[GTK_STATE_NORMAL],
			0, 0, pointer->width, pointer->height,
			GDK_RGB_DITHER_MAX, pointer->drawbuf, pointer->width);
  }
  gdk_threads_leave();
}


void* SimpleCanvas::process(void*)
{
  gdk_threads_enter();

  int    argc = 1;
  char** argv;
  argv    =   (char**) malloc(1*sizeof(char*));
  argv[0] = "SimpleCanvas";

  gtk_init(&argc, &argv);
  gdk_rgb_init();


  /*** Create GUI ***/
  /* main window  */

  pointer->window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
  gtk_widget_set_name(pointer->window, "Simple Image Canvas");

  pointer->vbox = gtk_vbox_new(FALSE, 0);
  gtk_container_add(GTK_CONTAINER(pointer->window), pointer->vbox);
  gtk_widget_show (pointer->vbox);

  gtk_signal_connect (GTK_OBJECT (pointer->window), "destroy",
		      GTK_SIGNAL_FUNC (gtk_exit), NULL);

  /** drawing area **/
  pointer->drawing_area = gtk_drawing_area_new();
  gtk_drawing_area_size(GTK_DRAWING_AREA (pointer->drawing_area),
			pointer->width, pointer->height);
  gtk_box_pack_start(GTK_BOX(pointer->vbox), pointer->drawing_area, TRUE, TRUE, 0);
  gtk_widget_show(pointer->drawing_area);
  
  /*** Signal handrings ***/
  /* drawing area */
  gtk_signal_connect(GTK_OBJECT(pointer->drawing_area), "expose_event",
		     (GtkSignalFunc)onDraw, NULL);

  gtk_widget_set_events(pointer->drawing_area, GDK_EXPOSURE_MASK
			 | GDK_LEAVE_NOTIFY_MASK
			 | GDK_BUTTON_PRESS_MASK
			 | GDK_POINTER_MOTION_MASK
			 | GDK_POINTER_MOTION_HINT_MASK);

  /* quit button */
  pointer->button = gtk_button_new_with_label("Quit");
  gtk_box_pack_start(GTK_BOX (pointer->vbox), pointer->button, FALSE, FALSE, 0); 
  gtk_signal_connect_object (GTK_OBJECT (pointer->button), "clicked",
			     GTK_SIGNAL_FUNC (gtk_widget_destroy),
			     GTK_OBJECT (pointer->window));
  gtk_widget_show (pointer->button);
  gtk_widget_show (pointer->window);

  gtk_main();

  gdk_threads_leave();
  return 0;
}

void SimpleCanvas::run()
{
  pthread_create(&thread, NULL, process, (void*)NULL);
  pthread_detach(thread);
}


void SimpleCanvas::stop()
{
  gtk_main_quit();
  //pthread_cancel(thread);
}
